import swarm.simtoolsgui.*;
import swarm.gui.*;
import swarm.space.*;
import swarm.activity.*;
import swarm.defobj.*;
import swarm.objectbase.*;
import swarm.*;
import java.util.*;

public class ConwayWorld2 extends ConwayLife2dImpl{
	ObserverSwarm observer;
	int xsize, ysize;

	//bz values
	int k1, k2;
	int constant_g, illness_n;
	int surrounding;
	int self_multiplation;
	int[] neighbors = new int[8];

	//border condition
	boolean border;

	//defects values and list
	public boolean defects;
	public List<Integer> defects_list = new ArrayList<Integer>();
	public int dx, dy;

	//rectangle values
	public int startx, starty;
	public int width, height;

	public ConwayWorld2(Zone aZone, int x, int y){
		super(aZone, x, y);
		
		//make probe map
		EmptyProbeMap probeMap;
		probeMap = new EmptyProbeMapImpl(aZone, this.getClass());
		probeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("defects",this.getClass()));
		probeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("dx",this.getClass()));
		probeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("dy",this.getClass()));
		probeMap.addProbe(Globals.env.probeLibrary.getProbeForMessage$inClass("addDefects",this.getClass()));
		probeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("startx",this.getClass()));
		probeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("starty",this.getClass()));
		probeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("width",this.getClass()));
		probeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("height",this.getClass()));
		probeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("defects_list",this.getClass()));
		probeMap.addProbe(Globals.env.probeLibrary.getProbeForMessage$inClass("showDefects",this.getClass()));

		Globals.env.probeLibrary.setProbeMap$For(probeMap,this.getClass());
		Globals.env.createArchivedProbeDisplay(this, "conway");
	}

	//set parameters of bz values
	public Object setParam(int K1, int K2, int constant_G, int illness_N, int Surrounding, int Self_multiplation) {
		k1 = K1;
		k2 = K2;
		constant_g = constant_G;
		illness_n = illness_N;
		surrounding = Surrounding;
		self_multiplation = Self_multiplation;
		this.setneighbors();

		return this;
	}

	//set neighbors (convert into 2digit)
	public void setneighbors(){
		int quotient;
		int remainder;
		quotient = surrounding;
		for (int i=0; i<7; i++){
			quotient = quotient/2;
			remainder = quotient%2;
			neighbors[7-i] = remainder;
		}
		neighbors[0] = quotient;
	}

	//set border condition
	public Object setBorder(boolean which){
		border = which;
		return this;
	}

	//set defects in coordinates
	public Object setDefect(boolean onoff, int x, int y){
		defects = onoff;
		dx = x;
		dy = y;
		return this;
	}

	//set rectangle in coordinates
	public Object setRectangle(int sx, int sy, int x, int y){
		startx = sx;
		starty = sy;
		width = x;
		height = y;
		return this;
	}

	public Object setSizeX$Y(int x, int y){
		xsize = x;
		ysize = y;
		return this;
	}
	public Object setObserver(ObserverSwarm anObject){
		observer = anObject;
		return this;
	}

	public Object swapColorAtX$Y(int x, int y){
		int newState;
		int oldState = this.getValueAtX$Y(x,y);

		newState = (oldState == 1) ? 2 : 0;
		this.putValue$atX$Y(newState,x,y);

		return this;
	}

	public Object eraseAll(){
		int i,j;
		for(i=0;i<xsize;i++){
			for(j=0;j<ysize;j++){
				this.putValue$atX$Y(0,i,j);
			}
		}
		return this;
	}

	//get state at x,y
	public int getStateAtX$Y(int x,int y){
		int state;
		state = this.getValue(x,y);

		if (state == 0 || state == 255) return 0;
		else if (state < this.illness_n) return 1;
		else return 2;
	}

	//get value arranged
	public int getValue(int x, int y){
		int value;
		int x1 = (x + xsize) % xsize;
		int y1 = (y + ysize) % ysize;

		if (border){ //considering border condition
			if (x==xsize||x==-1||y==ysize||y==-1) value = 0;
			else value = this.getValueAtX$Y(x,y);
		}
		else value = this.getValueAtX$Y(x1,y1);

		if (value==255) value = 0; //value 255 is defect or rectangle

		return value;
	}

	//get suma (infected)
	public int getSuma(int x, int y){
		int xm1, xp1, ym1, yp1;
		int suma = 0;
		xm1 = x-1;
		xp1 = x+1;
		ym1 = y-1;
		yp1 = y+1;

		if (neighbors[0]==1) suma += this.getStateAtX$Y(xm1, ym1)%2;
		if (neighbors[1]==1) suma += this.getStateAtX$Y(x,ym1)%2;
		if (neighbors[2]==1) suma += this.getStateAtX$Y(xp1,ym1)%2;

		if (neighbors[3]==1) suma += this.getStateAtX$Y(xm1,y)%2;
		if (neighbors[4]==1) suma += this.getStateAtX$Y(xp1, y)%2;

		if (neighbors[5]==1) suma += this.getStateAtX$Y(xm1, yp1)%2;
		if (neighbors[6]==1) suma += this.getStateAtX$Y(x, yp1)%2;
		if (neighbors[7]==1) suma += this.getStateAtX$Y(xp1, yp1)%2;

		return suma;
	}

	//get sumb (ill)
	public int getSumb(int x, int y){
		int xm1, xp1, ym1, yp1;
		int sumb = 0;
		xm1 = x-1;
		xp1 = x+1;
		ym1 = y-1;
		yp1 = y+1;

		if (neighbors[0]==1) sumb += this.getStateAtX$Y(xm1, ym1)/2;
		if (neighbors[1]==1) sumb += this.getStateAtX$Y(x,ym1)/2;
		if (neighbors[2]==1) sumb += this.getStateAtX$Y(xp1,ym1)/2;

		if (neighbors[3]==1) sumb += this.getStateAtX$Y(xm1,y)/2;
		if (neighbors[4]==1) sumb += this.getStateAtX$Y(xp1, y)/2;

		if (neighbors[5]==1) sumb += this.getStateAtX$Y(xm1, yp1)/2;
		if (neighbors[6]==1) sumb += this.getStateAtX$Y(x, yp1)/2;
		if (neighbors[7]==1) sumb += this.getStateAtX$Y(xp1, yp1)/2;

		return sumb;
	}

	//get sums (sum of values)
	public int getSums(int x, int y){
		int xm1, xp1, ym1, yp1;
		int sums = 0;
		xm1 = x-1;
		xp1 = x+1;
		ym1 = y-1;
		yp1 = y+1;

		if (neighbors[0]==1) sums += this.getValue(xm1,ym1);
		if (neighbors[1]==1) sums += this.getValue(x,ym1);
		if (neighbors[2]==1) sums += this.getValue(xp1,ym1);

		if (neighbors[3]==1) sums += this.getValue(xm1,y);
		if (neighbors[4]==1) sums += this.getValue(xp1, y);

		if (neighbors[5]==1) sums += this.getValue(xm1, yp1);
		if (neighbors[6]==1) sums += this.getValue(x, yp1);
		if (neighbors[7]==1) sums += this.getValue(xp1, yp1);

		sums += this.getValue(x,y);

		return sums;
	}

	public Object stepRule2(){
		int newState;
		int x,y;
		int suma, sumb, sums;
		int xm1, xp1, ym1, yp1;

		for(x=0; x<xsize; x++){
			for(y=0; y<ysize; y++){
				suma = this.getSuma(x,y); //infected
				sumb = this.getSumb(x,y); //ill
				sums = this.getSums(x,y); //sum of values

				switch(this.getStateAtX$Y(x,y)){ //healthy: 0 infect: 1 ill: 2 
				case 0:
					newState = suma/k1 + sumb/k2;
					break;
				case 2:
					newState = 0;
					break;
				case 1:
					newState = sums/(suma+sumb+1) + constant_g;
					if (newState >= this.illness_n) newState = illness_n;
					//System.out.println(newState);
					break;
				default:
					throw new IllegalArgumentException();
				}

				this.putValue$atX$Y(newState,x,y);
			}
		}
		this.updateLattice();
		return this;
	}

	//add defects on list
	public Object addDefects(){
		int dxdy;
		dxdy = 10000*dx+dy;

		if (defects_list.contains(dxdy)) {
			System.out.println("Remove a defect");
			defects_list.remove(defects_list.indexOf(dxdy));
		}
		else {
			System.out.println("Add a defect");
			defects_list.add(dxdy);
		}
		return this;
	}
	
	//to show the list of defects
	public void showDefects(){
		int tempdx, tempdy;

		if (defects) { //considering wheather defects are active or not
			System.out.println("Defects coordinate list");
			for (int i=0; i<defects_list.size(); i++){
				tempdx = defects_list.get(i)/10000;
				tempdy = defects_list.get(i)%10000;

				System.out.print(tempdx);
				System.out.print(", ");
				System.out.println(tempdy);
			}
		}
		else System.out.println("Defects are not active.");
	}

	//set defects
	public Object Defect(){
		int tempdx, tempdy;
		int dxdy;
		dxdy = 10000*dx+dy;

		//		if (defects_list.contains(dxdy)) defects_list.remove(defects_list.indexOf(dxdy));
		//		else defects_list.add(dxdy);

		if (defects) {
			for (int i=0; i<defects_list.size(); i++){
				tempdx = defects_list.get(i)/10000;
				tempdy = defects_list.get(i)%10000;
				this.putValue$atX$Y(255,tempdx,tempdy);
			}
		}
		this.updateLattice();

		return this;
	}

	//set rectangle
	public Object Rectangle(){
		int x, y;

		for(x=startx; x<(startx+width); x++){
			for(y=starty; y<(starty+height); y++){				
				this.putValue$atX$Y(255,x,y);
			}
		}
		this.updateLattice();

		return this;
	}

}